@@ -207,7 +207,7 @@ class MainWindow(QMainWindow):
         _export = QAction(self.style().standardIcon(QStyle.SP_FileDialogListView), "Capture viewport",self)
         file.addAction(_export)
         def trigger():
-            self.browser.page().runJavaScript("flask.savePNG();")
+            self.browser.page().runJavaScript("overlayUtils.savePNG();")
         _export.triggered.connect(trigger)
         
         _export = QAction(self.style().standardIcon(QStyle.SP_DirLinkIcon), "Export to static webpage",self)
@@ -432,6 +432,8 @@ class webEngine(QWebEngineView):
                 if not path[0] in state.keys():
                     return
                 if len(path) == 1:
+                    if path[0] not in state.keys():
+                        return
                     if isinstance(state[path[0]], list):
                         if isImg:
                             imgFiles += [s for s in state[path[0]]]
@@ -449,6 +451,8 @@ class webEngine(QWebEngineView):
                             state[path[0]] = "data/files/" + os.path.basename(state[path[0]])
                     return
                 else:
+                    if path[0] not in state.keys():
+                        return
                     if isinstance(state[path[0]], list):
                         for state_ in state[path[0]]:
                             addRelativePath_aux (state_, path[1:], isImg)
@@ -484,7 +488,7 @@ class webEngine(QWebEngineView):
             state = addRelativePath(json.loads(state), relativePath)
             
             with open(folderpath + "/project.tmap", "w") as f:
-                json.dump(state, f)
+                json.dump(state, f, indent=4)
             os.makedirs(os.path.join(folderpath,"data/images"), exist_ok=True )
             os.makedirs(os.path.join(folderpath,"data/files"), exist_ok=True )
             for image in imgFiles:
@@ -515,6 +519,8 @@ class webEngine(QWebEngineView):
         def addRelativePath(state, previouspath, newpath):
             def addRelativePath_aux (state, path):
                 if len(path) == 1:
+                    if path[0] not in state.keys():
+                        return
                     if isinstance(state[path[0]], list):
                         state[path[0]] = [getRel(previouspath, s, newpath) for s in state[path[0]]]
                     else:
@@ -597,17 +603,17 @@ class webEngine(QWebEngineView):
         #sys.exit()
 
     @pyqtSlot(str, str, result="QJsonObject")
-    def addCSV(self, path, folderpath):
-        print ("gui.addCSV", path, folderpath)
-        if (folderpath == ""):
-            folderpath = QFileDialog.getOpenFileName(self, 'Select a File')[0]
-        if not folderpath:
-            returnDict = {"dzi":None,"name":None}
+    def addCSV(self, path, csvpath):
+        print ("gui.addCSV", path, csvpath)
+        if (csvpath == ""):
+            csvpath = QFileDialog.getOpenFileName(self, 'Select a File')[0]
+        if not csvpath:
+            returnDict = {"markerFile":None}
             return returnDict
-        parts = Path(folderpath).parts
+        parts = Path(csvpath).parts
         if (self.app.basedir != parts[0]):
             if (not self.app.basedir == "C:\mnt\data\shared"):
-                reply = QMessageBox.warning(self, "Error", "All files must be in the same drive.")
+                QMessageBox.warning(self, "Error", "All files must be in the same drive.")
                 returnDict = {"markerFile":None}
                 return returnDict
             else:
@@ -618,12 +624,18 @@ class webEngine(QWebEngineView):
         imgPath = os.path.abspath(os.path.join(self.app.basedir, imgPath))
         
         relativePath = os.path.relpath(os.path.dirname(imgPath), path) 
+
+        print ("imgPath", imgPath)
+        print ("relativePath", relativePath)
+        print ("self.app.basedir", self.app.basedir)
+        print ("csvpath", csvpath)
+        print ("path", path)
+
         if ".." in relativePath:
-            reply = QMessageBox.question(self, "Error", "Impossible to add files from a parent folder.")
-            if reply == QMessageBox.Yes:
-                self.openImagePath(folderpath)
-            returnDict = {"dzi":None,"name":None}
+            QMessageBox.warning(self, "Error", "Impossible to add files from a parent folder.")
+            returnDict = {"markerFile":None}
             return returnDict
+
         returnDict = {
             "markerFile":{
                         "name": os.path.basename(imgPath),
@@ -635,20 +647,20 @@ class webEngine(QWebEngineView):
         return returnDict
     
     @pyqtSlot(str, str, result="QJsonObject")
-    def addLayer(self, path, folderpath):
-        if (folderpath == ""):
-            folderpath = QFileDialog.getOpenFileName(self, 'Select a File')[0]
-        if not folderpath:
+    def addLayer(self, path, layerpath):
+        if (layerpath == ""):
+            layerpath = QFileDialog.getOpenFileName(self, 'Select a File')[0]
+        if not layerpath:
             returnDict = {"dzi":None,"name":None}
             return returnDict
-        parts = Path(folderpath).parts
+        parts = Path(layerpath).parts
         print (self.app.basedir)
         if (self.app.basedir != parts[0]):
             if (not self.app.basedir == "C:\mnt\data\shared"):
                 reply = QMessageBox.question(self, "Error", "All layers must be in the same drive. Would you like to open this image only?")
                 reply = reply == QMessageBox.Yes
                 if reply:
-                    self.openImagePath(folderpath)
+                    self.openImagePath(layerpath)
                 returnDict = {"dzi":None,"name":None}
                 return returnDict
             else:
@@ -669,7 +681,7 @@ class webEngine(QWebEngineView):
         if ".." in relativePath:
             reply = QMessageBox.question(self, "Error", "Impossible to add layers from a parent folder. Would you like to open this image only?")
             if reply == QMessageBox.Yes:
-                self.openImagePath(folderpath)
+                self.openImagePath(layerpath)
             returnDict = {"dzi":None,"name":None}
             return returnDict
         returnDict = {
