@@ -80,24 +80,27 @@ flask.standalone.addLayer = function (filename) {
 }
 
 flask.standalone.saveProject = function () {
-    state = projectUtils.getActiveProject();
-    flask.standalone.backend.saveProject(JSON.stringify(state));
+    projectUtils.getActiveProject().then((state) => {
+        setTimeout(function() {
+            flask.standalone.backend.saveProject(JSON.stringify(state))
+        }, 300);
+    });
 };
 
 flask.standalone.exportToStatic = function () {
-    var state = projectUtils.getActiveProject();
-    var loadingModal =null;
-    setTimeout(function() {
-        loadingModal=interfaceUtils.loadingModal("Exporting to static web page")
-    },0);
-    setTimeout(function() {
-        flask.standalone.backend.exportToStatic(JSON.stringify(state));
+    projectUtils.getActiveProject().then((state) => {
+        var loadingModal =null;
         setTimeout(function() {
-            $(loadingModal).modal('hide');
-            interfaceUtils.alert("Exporting done.")
+            loadingModal=interfaceUtils.loadingModal("Exporting to static web page")
+        },0);
+        setTimeout(function() {
+            flask.standalone.backend.exportToStatic(JSON.stringify(state));
+            setTimeout(function() {
+                $(loadingModal).modal('hide');
+                interfaceUtils.alert("Exporting done.")
+            }, 500);
         }, 500);
-    }, 500);
-
+    })
 };
 
 flask.server = {}
@@ -109,31 +112,32 @@ flask.server.init = function () {
 
     interfaceUtils.addMenuItem(["File","Save project"],function(){
         var modalUID = "messagebox";
-        interfaceUtils.prompt("<b>Warning: only marker datasets converted into buttons will be saved.</b><br/><br/>Save project under the name:","NewProject","Save project")
-        .then((filename) => {
-            state = projectUtils.getActiveProject();
-            state.filename = filename;
-            if (filename.split('.').pop() != "tmap") {
-                filename = filename + ".tmap"
-            }
-            const queryString = window.location.search;
-            const urlParams = new URLSearchParams(queryString);
-            const path = urlParams.get('path')
-            $.ajax({
-                type: "POST",
-                url: "/" + filename + "?path=" + path,
-                // The key needs to match your method's input parameter (case-sensitive).
-                data: JSON.stringify(state),
-                contentType: "application/json; charset=utf-8",
-                dataType: "json",
-                success: function(data) {
-                    $('#loadingModal').modal('hide');
-                },
-                failure: function(errMsg) {
-                    $('#loadingModal').modal('hide');
-                    alert(errMsg);
+        projectUtils.getActiveProject().then((state) => {
+            interfaceUtils.prompt("Save project under the name:","NewProject","Save project")
+            .then((filename) => {
+                state.filename = filename;
+                if (filename.split('.').pop() != "tmap") {
+                    filename = filename + ".tmap"
                 }
-            });
+                const queryString = window.location.search;
+                const urlParams = new URLSearchParams(queryString);
+                const path = urlParams.get('path')
+                $.ajax({
+                    type: "POST",
+                    url: "/" + filename + "?path=" + path,
+                    // The key needs to match your method's input parameter (case-sensitive).
+                    data: JSON.stringify(state),
+                    contentType: "application/json; charset=utf-8",
+                    dataType: "json",
+                    success: function(data) {
+                        $('#loadingModal').modal('hide');
+                    },
+                    failure: function(errMsg) {
+                        $('#loadingModal').modal('hide');
+                        alert(errMsg);
+                    }
+                });
+            })
         })
     },true);
 
@@ -207,59 +211,24 @@ function toggleNavbar(turn_on = null) {
 /**
  * Save the current canvas as a PNG image
  */
- flask.savePNG=function() {
-    // Create an empty canvas element
-    var loading=interfaceUtils.loadingModal();
-    var canvas = document.createElement("canvas");
-    var ctx_osd = document.querySelector(".openseadragon-canvas canvas").getContext("2d");
-    var ctx_webgl = document.querySelector("#gl_canvas").getContext("webgl");
-    canvas.width = ctx_osd.canvas.width;
-    canvas.height = ctx_osd.canvas.height;
-    
-    // Copy the image contents to the canvas
-    var ctx = canvas.getContext("2d");
-    
-    ctx.drawImage(ctx_osd.canvas, 0, 0);
-    ctx.drawImage(ctx_webgl.canvas, 0, 0);
-    console.log(canvas.width,canvas.height);
-    var dataURL = canvas.toDataURL("image/png");
-    
-    var svgString = new XMLSerializer().serializeToString(document.querySelector('.openseadragon-canvas svg'));
-
-    var DOMURL = self.URL || self.webkitURL || self;
-    var img = new Image();
-    var svg = new Blob([svgString], {type: "image/svg+xml;charset=utf-8"});
-    var url = DOMURL.createObjectURL(svg);
-    img.onload = function() {
-        ctx.drawImage(img, 0, 0);
-        var png = canvas.toDataURL("image/png");
-           
-        var a = document.createElement("a"); //Create <a>
-        a.href = png; //Image Base64 Goes here
-        a.download = "TissUUmaps_capture.png"; //File name Here
-        a.click(); //Downloaded file
-        setTimeout(function(){$(loading).modal("hide");},500);
-        DOMURL.revokeObjectURL(png);
-    };
-    img.src = url;
-}
 
 // Child website:
 window.addEventListener("message", evt => {
     console.log("evt.data",evt.data) // "Question!"
+    // TODO: use overlayUtils.getCanvasPNG as a promise to get png image
     
     // Create an empty canvas element
     var canvas = document.createElement("canvas");
     var ctx_osd = document.querySelector(".openseadragon-canvas canvas").getContext("2d");
     var ctx_webgl = document.querySelector("#gl_canvas").getContext("webgl");
-    canvas.width = ctx_osd.canvas.width;
-    canvas.height = ctx_osd.canvas.height;
+    canvas.width = Math.min(ctx_osd.canvas.width, ctx_webgl.canvas.width);
+    canvas.height = Math.min(ctx_osd.canvas.height, ctx_webgl.canvas.height);
     
     // Copy the image contents to the canvas
     var ctx = canvas.getContext("2d");
     
-    ctx.drawImage(ctx_osd.canvas, 0, 0);
-    ctx.drawImage(ctx_webgl.canvas, 0, 0);
+    ctx.drawImage(ctx_osd.canvas, 0, 0, canvas.width, canvas.height);
+    ctx.drawImage(ctx_webgl.canvas, 0, 0, canvas.width, canvas.height);
     console.log(canvas.width,canvas.height);
     var dataURL = canvas.toDataURL("image/png");
     
