@@ -13,7 +13,6 @@ tmapp = {
     _url_suffix: "",
     _scrollDelay: 900,
     fixed_file: "",
-    mpp:0,
     slideFilename:"Main"
 }
 
@@ -24,7 +23,6 @@ tmapp.registerActions = function () {
     var op = tmapp["object_prefix"];
 
     interfaceUtils.listen(op + '_collapse_btn','click', function () { interfaceUtils.toggleRightPanel() },false);
-    /*interfaceUtils.listen(op + '_search','input', function () { markerUtils.hideRowsThatDontContain(); },false);*/
     interfaceUtils.listen(op + '_drawregions_btn','click', function () { regionUtils.regionsOnOff() },false);
     interfaceUtils.listen(op + '_export_regions','click', function () { regionUtils.exportRegionsToJSON() },false);
     interfaceUtils.listen(op + '_import_regions','click', function () { regionUtils.importRegionsFromJSON() },false);
@@ -32,7 +30,7 @@ tmapp.registerActions = function () {
     interfaceUtils.listen(op + '_fillregions_btn','click', function () { regionUtils.fillAllRegions(); },false);
     interfaceUtils.listen("capture_viewport","click",function(){overlayUtils.savePNG()},false)
     interfaceUtils.listen("plus-1-button","click",function(){interfaceUtils.generateDataTabUI()},false)
-    interfaceUtils.listen('save_project_menu', 'click', function() { projectUtils.saveProjectWindow() }, false);
+    interfaceUtils.listen('save_project_menu', 'click', function() { projectUtils.saveProject() }, false);
     interfaceUtils.listen('load_project_menu', 'click', function() { projectUtils.loadProjectFile() }, false);
     document.addEventListener("mousedown",function(){tmapp["ISS_viewer"].removeOverlay("ISS_marker_info");});
 
@@ -64,8 +62,6 @@ tmapp.init = function () {
     var vname = op + "_viewer";
     //init OSD viewer
     tmapp[vname] = OpenSeadragon(tmapp.options_osd);
-    //pixelate because we need the exact values of pixels
-    tmapp[vname].addHandler("tile-drawn", OSDViewerUtils.pixelateAtMaximumZoomHandler);
     // Disable keyboard hack
     tmapp[vname].innerTracker.keyHandler = null;
     tmapp[vname].innerTracker.keyDownHandler = null;
@@ -116,15 +112,13 @@ tmapp.init = function () {
     tmapp["ISS_viewer"].addHandler("animation-finish", function animationFinishHandler(event){
         console.log("animation-finish");
             d3.selectAll("." + regionUtils._drawingclass).selectAll('polyline').each(function(el) {
-                console.log(tmapp["ISS_viewer"].viewport.getZoom());
-                $(this).attr('stroke-width', 5* regionUtils._polygonStrokeWidth / tmapp["ISS_viewer"].viewport.getZoom());
+                $(this).attr('stroke-width', regionUtils._polygonStrokeWidth / tmapp["ISS_viewer"].viewport.getZoom());
             });
             d3.selectAll("." + regionUtils._drawingclass).selectAll('circle').each(function(el) {
-                console.log(tmapp["ISS_viewer"].viewport.getZoom());
                 $(this).attr('r', 10* regionUtils._handleRadius / tmapp["ISS_viewer"].viewport.getZoom());
             });
             d3.selectAll(".regionpoly").each(function(el) {
-                $(this).attr('stroke-width', 5* regionUtils._polygonStrokeWidth / tmapp["ISS_viewer"].viewport.getZoom());
+                $(this).attr('stroke-width', regionUtils._polygonStrokeWidth / tmapp["ISS_viewer"].viewport.getZoom());
             });
     });
     
@@ -135,21 +129,7 @@ tmapp.init = function () {
         });
         elt.classList.add("d-none");
     }
-
-    if (tmapp.mpp != 0) {
-        tmapp[vname].scalebar({
-            pixelsPerMeter: tmapp.mpp ? (1e6 / tmapp.mpp) : 0,
-            xOffset: 200,
-            yOffset: 10,
-            barThickness: 3,
-            color: '#555555',
-            fontColor: '#333333',
-            backgroundColor: 'rgba(255, 255, 255, 0.5)',
-            sizeAndTextRenderer: OpenSeadragon.ScalebarSizeAndTextRenderer.METRIC_LENGTH,
-            location: OpenSeadragon.ScalebarLocation.BOTTOM_RIGHT
-        });
-    }
-    //document.getElementById('cancelsearch-moving-button').addEventListener('click', function(){ markerUtils.showAllRows("moving");}); 
+    
     filterUtils.initFilters();
     if (window.hasOwnProperty("glUtils")) {
         console.log("Using GPU-based marker drawing (WebGL canvas)")
@@ -178,9 +158,18 @@ tmapp.options_osd = {
     constrainDuringPan: true,
     visibilityRatio: 0.85,
     showNavigationControl: false,
-    maxImageCacheCount:500,
+    maxImageCacheCount:2000,
     imageSmoothingEnabled:false,
-    preserveImageSizeOnResize: true
+    preserveImageSizeOnResize: true,
+    gestureSettingsUnknown: {
+        flickEnabled: false
+    },
+    gestureSettingsTouch: {
+        flickEnabled: false
+    },
+    gestureSettingsPen: {
+        flickEnabled: false
+    }
 }
 
 function toggleFullscreen() {
