@@ -7,8 +7,8 @@ glUtils = {
     _initialized: false,
     _imageSize: [1, 1],
     _viewportRect: [0, 0, 1, 1],
-    _options: {antialias: false, premultipliedAlpha: false, preserveDrawingBuffer: true},
-
+    _options: {antialias: false, premultipliedAlpha: true, preserveDrawingBuffer: true},
+    _markershapes: "misc/markershapes.png",
     // WebGL resources
     _programs: {},
     _buffers: {},
@@ -22,6 +22,7 @@ glUtils = {
     _markerScaleFactor: {},      // {uid: float}
     _markerScalarPropertyName: {},  // {uid: string, ...}
     _markerOpacity: {},          // {uid: alpha, ...}
+    _markerOutline: {},          // {uid: boolean, ...}
     _useColorFromMarker: {},     // {uid: boolean, ...}
     _useColorFromColormap: {},   // {uid: boolean, ...}
     _useScaleFromMarker: {},     // {uid: boolean, ...}
@@ -42,6 +43,8 @@ glUtils = {
     _pickedMarker: [-1, -1],
     _showColorbar: true,
     _showMarkerInfo: true,
+    _resolutionScale: 1.0,        // If this is set to below 1.0, the WebGL output will be upscaled
+    _resolutionScaleActual: 1.0,  // Automatic scaling factor computed from glUtils._resolutionScale
     _piechartPalette: ["#fff100", "#ff8c00", "#e81123", "#ec008c", "#68217a", "#00188f", "#00bcf2", "#00b294", "#009e49", "#bad80a"]
 }
 
@@ -139,6 +142,7 @@ glUtils._markersVS = `
 glUtils._markersFS = `
     precision mediump float;
 
+    uniform bool u_markerOutline;
     uniform bool u_usePiechartFromMarker;
     uniform bool u_alphaPass;
     uniform sampler2D u_shapeAtlas;
@@ -177,7 +181,13 @@ glUtils._markersFS = `
         vec2 uv = (gl_PointCoord.xy - 0.5) * UV_SCALE + 0.5;
         uv = (uv + v_shapeOrigin) * (1.0 / SHAPE_GRID_SIZE);
 
+        // Sample shape texture in which the blue channel encodes alpha and the
+        // red and green channels encode grayscale for marker shape with and
+        // without outline, respectively
         vec4 shapeColor = texture2D(u_shapeAtlas, uv, -0.5);
+        shapeColor = u_markerOutline ? shapeColor.rrrb : shapeColor.gggb;
+
+        // This bias avoids minified markers with outline becoming too dark
         float shapeColorBias = max(0.0, 1.0 - v_shapeSize * 0.2);
         shapeColor.rgb = clamp(shapeColor.rgb + shapeColorBias, 0.0, 1.0);
 
@@ -267,12 +277,13 @@ glUtils._pickingVS = `
             if (abs(uv.x - 0.5) > 0.5 || abs(uv.y - 0.5) > 0.5) DISCARD_VERTEX;
 
             // Do fine-grained inside/outside test by sampling the shape texture
+            // with alpha encoded in the blue channel
             vec2 shapeOrigin = vec2(0.0);
             shapeOrigin.x = mod((shapeID + 0.00001) * 255.0 - 1.0, SHAPE_GRID_SIZE);
             shapeOrigin.y = floor(((shapeID + 0.00001) * 255.0 - 1.0) / SHAPE_GRID_SIZE);
             uv = (uv - 0.5) * UV_SCALE + 0.5;
             uv = (uv + shapeOrigin) * (1.0 / SHAPE_GRID_SIZE);
-            if (texture2D(u_shapeAtlas, uv).r < 0.5) DISCARD_VERTEX;
+            if (texture2D(u_shapeAtlas, uv).b < 0.5) DISCARD_VERTEX;
 
             // Also do a quick alpha-test to avoid picking non-visible markers
             if (a_opacity * u_markerOpacity <= 0.0) DISCARD_VERTEX
@@ -407,11 +418,13 @@ glUtils.loadMarkers = function(uid) {
     const useShapeFromMarker = dataUtils.data[uid]["_shape_col"] != null;
     const numShapes = Object.keys(markerUtils._symbolStrings).length;
     let shapeIndex = 0;
-    
+
     const opacityPropertyName = dataUtils.data[uid]["_opacity_col"];
     const useOpacityFromMarker = dataUtils.data[uid]["_opacity_col"] != null;
     const markerOpacityFactor = dataUtils.data[uid]["_opacity"];
 
+    const markerOutline = !dataUtils.data[uid]["_no_outline"];
+
     // Additional info about the vertex format
     const NUM_COMPONENTS_PER_MARKER = 8;
     const NUM_BYTES_PER_MARKER = NUM_COMPONENTS_PER_MARKER * 4;
@@ -533,17 +546,18 @@ glUtils.loadMarkers = function(uid) {
 
     // Update marker info and LUT + colormap textures
     glUtils._numPoints[uid] = numPoints * numSectors;
-    glUtils._markerOpacity[uid] = markerOpacityFactor;
     glUtils._markerScalarRange[uid] = scalarRange;
     glUtils._markerScalarPropertyName[uid] = scalarPropertyName;
     glUtils._markerScaleFactor[uid] = markerScaleFactor;
-    glUtils._colorscaleName[uid] = colorscaleName;
+    glUtils._markerOpacity[uid] = markerOpacityFactor;
+    glUtils._markerOutline[uid] = markerOutline;
     glUtils._useColorFromMarker[uid] = useColorFromMarker;
     glUtils._useColorFromColormap[uid] = useColorFromColormap;
     glUtils._useScaleFromMarker[uid] = useScaleFromMarker;
     glUtils._useOpacityFromMarker[uid] = useOpacityFromMarker;
     glUtils._usePiechartFromMarker[uid] = usePiechartFromMarker;
     glUtils._useShapeFromMarker[uid] = useShapeFromMarker;
+    glUtils._colorscaleName[uid] = colorscaleName;
     if (useColorFromColormap) {
         glUtils._updateColorScaleTexture(gl, uid, glUtils._textures[uid + "_colorscale"]);
     }
@@ -564,7 +578,9 @@ glUtils.deleteMarkers = function(uid) {
     delete glUtils._numPoints[uid];
     delete glUtils._markerScaleFactor[uid];
     delete glUtils._markerScalarRange[uid];
+    delete glUtils._markerScalarPropertyName[uid];
     delete glUtils._markerOpacity[uid];
+    delete glUtils._markerOutline[uid];
     delete glUtils._useColorFromMarker[uid];
     delete glUtils._useColorFromColormap[uid];
     delete glUtils._useScaleFromMarker[uid];
@@ -836,7 +852,7 @@ glUtils._createMarkerWebGLCanvas = function() {
     const canvas = document.createElement("canvas");
     canvas.id = "gl_canvas";
     canvas.width = "1"; canvas.height = "1";
-    canvas.style = "position:relative; pointer-events:none; z-index: 12;";
+    canvas.style = "position:relative; pointer-events:none; z-index: 12; width: 100%; height: 100%";
     return canvas;
 }
 
@@ -902,6 +918,7 @@ glUtils.drawColorPass = function(gl, viewportTransform, markerScaleAdjusted) {
         gl.uniform1f(gl.getUniformLocation(program, "u_globalMarkerScale"), glUtils._globalMarkerScale * glUtils._markerScaleFactor[uid]);
         gl.uniform2fv(gl.getUniformLocation(program, "u_markerScalarRange"), glUtils._markerScalarRange[uid]);
         gl.uniform1f(gl.getUniformLocation(program, "u_markerOpacity"), glUtils._markerOpacity[uid]);
+        gl.uniform1i(gl.getUniformLocation(program, "u_markerOutline"), glUtils._markerOutline[uid]);
         gl.uniform1i(gl.getUniformLocation(program, "u_useColorFromMarker"), glUtils._useColorFromMarker[uid]);
         gl.uniform1i(gl.getUniformLocation(program, "u_useColorFromColormap"), glUtils._useColorFromColormap[uid]);
         gl.uniform1i(gl.getUniformLocation(program, "u_usePiechartFromMarker"), glUtils._usePiechartFromMarker[uid]);
@@ -1021,7 +1038,7 @@ glUtils.draw = function() {
     let markerScaleAdjusted = glUtils._markerScale;
     if (glUtils._useMarkerScaleFix) markerScaleAdjusted *= (gl.canvas.height / 900.0);
 
-    gl.clearColor(0.5, 0.5, 0.5, 0.0);
+    gl.clearColor(0.0, 0.0, 0.0, 0.0);
     gl.clear(gl.COLOR_BUFFER_BIT);
     gl.viewport(0, 0, gl.canvas.width, gl.canvas.height);
 
@@ -1037,7 +1054,8 @@ glUtils.draw = function() {
 glUtils.pick = function(event) {
     if (event.quick) {
         glUtils._pickingEnabled = true;
-        glUtils._pickingLocation = [event.position.x, event.position.y];
+        glUtils._pickingLocation = [event.position.x * glUtils._resolutionScaleActual,
+                                    event.position.y * glUtils._resolutionScaleActual];
         glUtils.draw();  // This will update the value of glUtils._pickedMarker
 
         const pickedMarker = glUtils._pickedMarker;
@@ -1089,9 +1107,19 @@ glUtils.resize = function() {
     const gl = canvas.getContext("webgl", glUtils._options);
 
     const op = tmapp["object_prefix"];
-    const newSize = tmapp[op + "_viewer"].viewport.containerSize;
-    gl.canvas.width = newSize.x;
-    gl.canvas.height = newSize.y;
+    const width = tmapp[op + "_viewer"].viewport.containerSize.x * window.devicePixelRatio;
+    const height = tmapp[op + "_viewer"].viewport.containerSize.y * window.devicePixelRatio;
+
+    glUtils._resolutionScaleActual = glUtils._resolutionScale;
+    if (Math.max(width, height) * glUtils._resolutionScale >= 4096.0) {
+        // A too large WebGL canvas can lead to misalignment between the WebGL
+        // markers and the OSD image layers, so here the resolution scaling
+        // factor is adjusted to restrict the canvas size to a safe value
+        glUtils._resolutionScaleActual *= 4096.0 / (Math.max(width, height) * glUtils._resolutionScale);
+    }
+
+    gl.canvas.width = width * glUtils._resolutionScaleActual;
+    gl.canvas.height = height * glUtils._resolutionScaleActual;
 }
 
 
@@ -1117,7 +1145,7 @@ glUtils._restoreLostContext = function(event) {
     // Restore shared WebGL objects
     glUtils._programs["markers"] = glUtils._loadShaderProgram(gl, glUtils._markersVS, glUtils._markersFS);
     glUtils._programs["picking"] = glUtils._loadShaderProgram(gl, glUtils._pickingVS, glUtils._pickingFS);
-    glUtils._textures["shapeAtlas"] = glUtils._loadTextureFromImageURL(gl, "/static/misc/markershapes.png");
+    glUtils._textures["shapeAtlas"] = glUtils._loadTextureFromImageURL(gl, glUtils._markershapes);
 
     // Restore per-markers WebGL objects
     for (let [uid, numPoints] of Object.entries(glUtils._numPoints)) {
@@ -1147,7 +1175,7 @@ glUtils.init = function() {
 
     this._programs["markers"] = this._loadShaderProgram(gl, this._markersVS, this._markersFS);
     this._programs["picking"] = this._loadShaderProgram(gl, this._pickingVS, this._pickingFS);
-    this._textures["shapeAtlas"] = this._loadTextureFromImageURL(gl, "/static/misc/markershapes.png");
+    this._textures["shapeAtlas"] = this._loadTextureFromImageURL(gl, glUtils._markershapes);
 
     this._createColorbarCanvas();  // The colorbar is drawn separately in a 2D-canvas
 
