@@ -234,7 +234,7 @@ class _SlideCache(object):
             mpp_x = osr.properties[openslide.PROPERTY_NAME_MPP_X]
             mpp_y = osr.properties[openslide.PROPERTY_NAME_MPP_Y]
             slide.properties = osr.properties
-            slide.mpp = (float(mpp_x) + float(mpp_y)) / 2
+            slide.mpp = float(mpp_x)
         except (KeyError, ValueError):
             try:
                 if osr.properties["tiff.ResolutionUnit"] == "centimetre":
@@ -408,7 +408,7 @@ def tmapFile(filename):
         else:
             abort(404)
         if "plugins" in state.keys():
-            plugins = state["plugins"]
+            plugins = []
         else:
             plugins = app.config["PLUGINS"]
 
@@ -512,8 +512,8 @@ def dzi_asso(path):
 def tile(path, level, col, row, format):
     completePath = os.path.join(app.basedir, path)
     if os.path.isfile( f"{completePath}_files/{level}/{col}_{row}.{format}"):
-        directory = os.path.dirname(f"{completePath}_files/{level}/{col}_{row}.{format}")
-        filename = os.path.basename(f"{completePath}_files/{level}/{col}_{row}.{format}")
+        directory = f"{completePath}_files/{level}/"
+        filename = f"{col}_{row}.{format}"
         return send_from_directory(directory, filename)
     slide = _get_slide(path)
     format = format.lower()
