@@ -239,6 +239,7 @@ var data_obj = dataUtils.data[data_id];
     if (radios["cb_gr"].checked) { // Color by group
         data_obj["_cb_col"]=null;
         data_obj["_cb_cmap"]=null;
+        data_obj["_cb_gr_dict"]=inputs["cb_gr_dict"].value;
     }
     else if (radios["cb_col"].checked) { // Color by marker
         if (inputs["cb_col"].value != "null") {
@@ -256,6 +257,7 @@ var data_obj = dataUtils.data[data_id];
     }
     // Use piecharts column
     data_obj["_pie_col"]=(radios["pie_check"].checked ? inputs["pie_col"].value : null);
+    data_obj["_pie_dict"]=inputs["pie_dict"].value;
     if (data_obj["_pie_col"]=="null") {
         interfaceUtils.alert("No piechart column selected. Impossible to update view.");return;
     }
@@ -270,8 +272,14 @@ var data_obj = dataUtils.data[data_id];
     if (data_obj["_shape_col"]=="null") {
         interfaceUtils.alert("No shape column selected. Impossible to update view.");return;
     }
-    // Marker opacity
+    // Use opacity column
+    data_obj["_opacity_col"]=(radios["opacity_check"].checked ? inputs["opacity_col"].value : null);
+    if (data_obj["_opacity_col"]=="null") {
+        interfaceUtils.alert("No opacity column selected. Impossible to update view.");return;
+    }
     data_obj["_opacity"]=inputs["opacity"].value;
+    // Tooltip
+    data_obj["_tooltip_fmt"]=inputs["tooltip_fmt"].value;
     
     //this function veryfies if a tree with these features exist and doesnt recreate it
     dataUtils.makeQuadTrees(data_id);
@@ -314,7 +322,7 @@ dataUtils.createMenuFromCSV = function(data_id,datumExample) {
     namesymbols.forEach((drop)=>{
         if(drop=="cb_cmap") return; //if its colormaps dont fill it with csv but with d3 luts which are already there
         if(drop=="shape_fixed") return; //if its shapes dont fill it with csv but with shape symbols which are already there
-        if(drop=="scale_factor" || drop=="shape_gr_dict" || drop=="cb_gr_dict" || drop=="opacity") return; //not dropdowns
+        if(drop=="scale_factor" || drop=="shape_gr_dict" || drop=="cb_gr_dict" || drop=="opacity" || drop=="pie_dict" || drop=="tooltip_fmt") return; //not dropdowns
         if (!alldrops[drop]) return;
         alldrops[drop].innerHTML = "";
         var option = document.createElement("option");
@@ -348,6 +356,7 @@ dataUtils.readCSV = function(data_id, thecsv, options) {
     data_obj["_csv_header"] = null;
     data_obj["_csv_path"] = thecsv;
     if (options != undefined) {
+        data_obj["_csv_path"] = options.path;
         data_obj["expectedHeader"] = options.expectedHeader;
         data_obj["expectedRadios"] = options.expectedRadios;
         data_obj["fromButton"] = options.fromButton;
